@@ -179,6 +179,33 @@ gail_menu_item_new (GtkWidget *widget)
   return accessible;
 }
 
+GList *
+get_children (GtkWidget *submenu)
+{
+  GList *children;
+
+  children = gtk_container_get_children (GTK_CONTAINER (submenu));
+  if (g_list_length (children) == 0)
+    {
+      /*
+       * If menu is empty it may be because the menu items are created only 
+       * on demand. For example, in gnome-panel the menu items are created
+       * only when "show" signal is emitted on the menu.
+       *
+       * The following hack forces the menu items to be created.
+       */
+      if (!GTK_WIDGET_VISIBLE (submenu))
+        {
+          GTK_WIDGET_SET_FLAGS (submenu, GTK_VISIBLE);
+          g_signal_emit_by_name (submenu, "show");
+          GTK_WIDGET_UNSET_FLAGS (submenu, GTK_VISIBLE);
+        }
+      g_list_free (children);
+      children = gtk_container_get_children (GTK_CONTAINER (submenu));
+    }
+  return children;
+}
+
 /*
  * If a menu item has a submenu return the items of the submenu as the 
  * accessible children; otherwise expose no accessible children.
@@ -201,7 +228,7 @@ gail_menu_item_get_n_children (AtkObject* obj)
     {
       GList *children;
 
-      children = gtk_container_get_children (GTK_CONTAINER (submenu));
+      children = get_children (submenu);
       count = g_list_length (children);
       g_list_free (children);
     }
@@ -229,7 +256,7 @@ gail_menu_item_ref_child (AtkObject *obj,
       GList *children;
       GList *tmp_list;
 
-      children = gtk_container_get_children (GTK_CONTAINER (submenu));
+      children = get_children (submenu);
       tmp_list = g_list_nth (children, i);
       if (!tmp_list)
         {
@@ -355,7 +382,9 @@ gail_menu_item_get_keybinding (AtkAction *action,
   /*
    * This function returns a string of the form A;B;C where
    * A is the keybinding for the widget; B is the keybinding to traverse
-   * from the menubar and C is the accelerator
+   * from the menubar and C is the accelerator.
+   * The items in the keybinding to traverse from the menubar are separated
+   * by ":".
    */
   GailMenuItem  *gail_menu_item;
   gchar *keybinding = NULL;
@@ -415,7 +444,10 @@ gail_menu_item_get_keybinding (AtkAction *action,
               if (key_val != GDK_VoidSymbol)
                 {
                   key = gtk_accelerator_name (key_val, mnemonic_modifier);
-                  temp_keybinding = g_strconcat (key, full_keybinding, NULL);
+                  if (full_keybinding)
+                    temp_keybinding = g_strconcat (key, ":", full_keybinding, NULL);
+                  else 
+                    temp_keybinding = g_strconcat (key, NULL);
                   if (temp_item == item)
                     {
                       item_keybinding = g_strdup (key); 
